#include <hpgcc49.h>
#ifndef HPAPINE
#include <fsystem.h>
extern U32 syscallArg0(U32 index);
#endif

#define MINBLOCK 8
#define MAXBLOCK 12

int main()
{
	unsigned int blocksize;
	unsigned int filesize, i, j, n;
	unsigned int readp[MAXBLOCK + 1], writep[MAXBLOCK + 1];
	char *block, *filename = "sdbench.dat", *ofile = "sdbench.xls";
	int start, end;
#ifndef HPAPINE
	FS_FILE *fp;
#else
	FILE *fp;
#endif
	FILE *ofp;

	filesize = (int) sat_pop_real();
	filesize *= 1024 * 1024;	//megabytes

#ifndef HPAPINE
	FSOpen(filename, FSMODE_WRITE | FSMODE_READ | FSMODE_MODIFY, &fp);
	if (fp == NULL) {
#else
	if ((fp = fopen(filename, "wb+")) == NULL) {
#endif
		char errormsg[50];
		sprintf(errormsg, "Cannot open: %s", filename);
		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return (0);
	}

	blocksize = pow(2, MAXBLOCK);

	if ((block = malloc(blocksize * sizeof(char))) == NULL) {
		char errormsg[50];
		sprintf(errormsg,
				"Failed to allocate %d bytes of memory for blocksize",
				blocksize * (int) sizeof(char));
		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return (0);
	}

	sys_slowOff();
	clear_screen();
	printf("SD Bench\n");
	printf("--------\n\n");
	printf("Creating %d MB file...", filesize / 1024 / 1024);

	for (j = 0; j < blocksize; j++)
		block[j] = 0;

	for (i = 0; i < (filesize / blocksize) + 1; i++) {
#ifndef HPAPINE
		FSWrite(block, blocksize, fp);
#else
		fwrite(block, blocksize, 1, fp);
#endif
	}

	free(block);

	for (i = MINBLOCK; i <= MAXBLOCK; i++) {
		blocksize = pow(2, i);
		n = filesize / blocksize;

		if ((block = malloc(blocksize * sizeof(char))) == NULL) {
			char errormsg[50];
			sprintf(errormsg,
					"Failed to allocate %d bytes of memory for blocksize",
					blocksize * (int) sizeof(char));
			sat_stack_push_string(errormsg);
			sat_push_real(1);
			return (0);
		}

		for (j = 0; j < blocksize; j++)
			block[j] = 65;

		gotoxy(0, 3);
		for (j = 3; j < 13; j++) {
			gotoxy(0, j);
			printf("                              ");
		}
		gotoxy(0, 3);

		printf("Blocksize:         %d\n", blocksize);
		printf("Number of blocks:  %d\n\n", n);

#ifndef HPAPINE
		FSSeek(fp, 0, 0);
#else
		fseek(fp, 0, 0);
#endif
		start = sys_RTC_seconds();
		for (j = 0; j < n; j++) {
#ifndef HPAPINE
			FSWrite(block, blocksize, fp);
#else
			fwrite(block, blocksize, 1, fp);
#endif
		}
		end = sys_RTC_seconds();
		printf("Write time(s):     %d\n", end - start);
		printf("Blocks/second:     ");
		if (end - start)
			printf("%d\n", n / (end - start));
		else
			printf("NAN\n");
		printf("Bytes/second:      ");
		if (end - start) {
			printf("%d\n", n * blocksize / (end - start));
			writep[i] = ((n * blocksize) / (end - start));
		} else {
			printf("NAN\n");
			writep[i] = 0;
		}

#ifndef HPAPINE
		FSSeek(fp, 0, 0);
#else
		fseek(fp, 0, 0);
#endif
		start = sys_RTC_seconds();
		for (j = 0; j < n; j++) {
#ifndef HPAPINE
			FSRead(block, blocksize, fp);
#else
			fread(block, blocksize, 1, fp);
#endif
		}
		end = sys_RTC_seconds();
		printf("\nRead time(s):      %d\n", end - start);
		printf("Blocks/second:     ");
		if (end - start)
			printf("%d\n", n / (end - start));
		else
			printf("NAN\n");
		printf("Bytes/second:      ");
		if (end - start) {
			printf("%d", n * blocksize / (end - start));
			readp[i] = ((n * blocksize) / (end - start));
		} else {
			printf("NAN");
			readp[i] = 0;
		}

		free(block);
	}

#ifndef HPAPINE
	FSCloseAndDelete(fp);
	syscallArg0(19);
#else
	fclose(fp);
#endif

	if ((ofp = fopen(ofile, "w")) == NULL) {
		char errormsg[50];
		sprintf(errormsg, "Cannot open: %s", ofile);
		sat_stack_push_string(errormsg);
		sat_push_real(1);
		return (0);
	}

	fprintf(ofp, "\tWrite\tRead\n");
	for (i = MAXBLOCK; i >= MINBLOCK; i--) {
		blocksize = pow(2, i);
		fprintf(ofp, "%d\t%d\t%d\n", blocksize, writep[i], readp[i]);
	}
	fclose(ofp);

	sat_push_real(0);
	sys_slowOn();
	beep();
	WAIT_CANCEL;
	return (0);
}
